/*
 * $Revision: 43401 $
 * $Date: 2007-04-05 11:50:40 -0700 (Thu, 05 Apr 2007) $
 *
 * Copyright (C) 1999-2006 Jive Software. All rights reserved.
 *
 * This software is the proprietary information of Jive Software. Use is subject to license terms.
 *
 */

/*
 * Copyright 2004 Sun Microsystems, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package org.jivesoftware.plugins.instantFeeds;

import com.sun.syndication.feed.synd.SyndFeed;
import com.sun.syndication.fetcher.FetcherEvent;
import com.sun.syndication.fetcher.FetcherException;
import com.sun.syndication.fetcher.impl.*;
import com.sun.syndication.io.*;
import org.apache.commons.httpclient.*;
import org.apache.commons.httpclient.auth.AuthScope;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.protocol.Protocol;

import java.io.IOException;
import java.io.InputStream;
import java.net.*;
import java.util.zip.GZIPInputStream;

/**
 * Feed fetcher implementation that accepts SSL certs that are expired / invalid
 * and that properly handles HTTP basic auth.
 */
public class InstantFeedsFeedFetcher extends AbstractFeedFetcher {

    private FeedFetcherCache feedInfoCache;
    private HttpClientFeedFetcher.CredentialSupplier credentialSupplier;

    public InstantFeedsFeedFetcher() {
        super();
    }

    /**
     * @param cache the cache to use
     */
    public InstantFeedsFeedFetcher(FeedFetcherCache cache) {
        this();
        setFeedInfoCache(cache);
    }

    public InstantFeedsFeedFetcher(FeedFetcherCache cache,
            HttpClientFeedFetcher.CredentialSupplier credentialSupplier)
    {
        this(cache);
        setCredentialSupplier(credentialSupplier);
    }

    /**
     * @return the feedInfoCache.
     */
    public synchronized FeedFetcherCache getFeedInfoCache() {
        return feedInfoCache;
    }

    /**
     * @param feedInfoCache the feedInfoCache to set
     */
    public synchronized void setFeedInfoCache(FeedFetcherCache feedInfoCache) {
        this.feedInfoCache = feedInfoCache;
    }

    /**
     * @return Returns the credentialSupplier.
     */
    public synchronized HttpClientFeedFetcher.CredentialSupplier getCredentialSupplier() {
        return credentialSupplier;
    }

    /**
     * @param credentialSupplier The credentialSupplier to set.
     */
    public synchronized void setCredentialSupplier(
            HttpClientFeedFetcher.CredentialSupplier credentialSupplier)
    {
        this.credentialSupplier = credentialSupplier;
    }

    public SyndFeed retrieveFeed(URL feedUrl)
            throws IllegalArgumentException, IOException, FeedException, FetcherException
    {
        return retrieveFeed(feedUrl, -1);
    }

    /**
     * @param feedUrl the url to fetch
     * @param timeout number of milliseconds to timeout after
     * @return an RSS or Atom feed
     * @throws IllegalArgumentException exception
     * @throws IOException exception
     * @throws FeedException exception
     * @throws FetcherException exception
     * @see com.sun.syndication.fetcher.FeedFetcher#retrieveFeed(java.net.URL)
     */
    @SuppressWarnings("deprecation")
    public SyndFeed retrieveFeed(URL feedUrl, int timeout)
            throws IllegalArgumentException, IOException, FeedException, FetcherException
    {
        if (feedUrl == null) {
            throw new IllegalArgumentException("null is not a valid URL");
        }

        HttpClient client = getHttpClient(feedUrl, timeout);

        String urlStr = feedUrl.toString();
        FeedFetcherCache cache = getFeedInfoCache();
        if (cache != null) {
            // retrieve feed

            HttpMethod method = getHttpGetMethod(feedUrl);

            method.addRequestHeader("Accept-Encoding", "gzip");
            try {
                if (isUsingDeltaEncoding()) {
                    method.setRequestHeader("A-IM", "feed");
                }

                // get the feed info from the cache
                // Note that syndFeedInfo will be null if it is not in the cache
                SyndFeedInfo syndFeedInfo = cache.getFeedInfo(feedUrl);
                if (syndFeedInfo != null) {
                    method.setRequestHeader("If-None-Match", syndFeedInfo.getETag());

                    if (syndFeedInfo.getLastModified() instanceof String) {
                        method.setRequestHeader("If-Modified-Since",
                                (String) syndFeedInfo.getLastModified());
                    }
                }

                method.setFollowRedirects(true);

                int statusCode = client.executeMethod(method);
                fireEvent(FetcherEvent.EVENT_TYPE_FEED_POLLED, urlStr);
                handleErrorCodes(statusCode);

                SyndFeed feed = getFeed(syndFeedInfo, urlStr, method, statusCode);

                syndFeedInfo = buildSyndFeedInfo(feedUrl, urlStr, method, feed, statusCode);

                cache.setFeedInfo(new URL(urlStr), syndFeedInfo);

                // the feed may have been modified to pick up cached values
                // (eg - for delta encoding)
                feed = syndFeedInfo.getSyndFeed();

                return feed;
            }
            finally {
                method.releaseConnection();
            }
        }
        else {
            // cache is not in use
            HttpMethod method = new GetMethod(urlStr);
            try {
                method.setFollowRedirects(true);

                int statusCode = client.executeMethod(method);
                fireEvent(FetcherEvent.EVENT_TYPE_FEED_POLLED, urlStr);
                handleErrorCodes(statusCode);

                return getFeed(null, urlStr, method, statusCode);
            }
            finally {
                method.releaseConnection();
            }
        }
    }


    private HttpMethod getHttpGetMethod(URL feedUrl) {

        // for HTTPS requests use a relative URL so that we use the specific SSL factory we created above
        if (isHTTPS(feedUrl)) {
            String path = feedUrl.getPath();
            String query = feedUrl.getQuery();
            if (query != null && !"".equals(query)) {
                path += "?" + query;
            }
            return new GetMethod(path);
        }
        else {
            return new GetMethod(feedUrl.toExternalForm());
        }
    }


    @SuppressWarnings("deprecation")
    private HttpClient getHttpClient(URL feedUrl, int timeout) {
        HttpClient client = new HttpClient();
        HttpConnectionManager conManager = client.getHttpConnectionManager();
        if (timeout > 0) {
            conManager.getParams().setParameter("http.connection.timeout", timeout);
            conManager.getParams().setParameter("http.socket.timeout", timeout);
        }

        // use an ssl socket factory that doesn't care about certs
        if (isHTTPS(feedUrl)) {
            int port = (feedUrl.getPort() > -1 ? feedUrl.getPort() : 443);
            Protocol myhttps = new Protocol("https", new InstantFeedsDummySSLSocketFactory(), port);
            client.getHostConfiguration().setHost(feedUrl.getHost(), port, myhttps);
        }

        // check to see if they are requesting an http basic auth feed
        if (feedUrl.getUserInfo() != null && getCredentialSupplier() == null) {
            setCredentialSupplier(new BasicAuthCredentials(feedUrl.getUserInfo()));
        }

        if (getCredentialSupplier() != null) {
            client.getParams().setAuthenticationPreemptive(true);
            client.getState().setCredentials(
                    new AuthScope(feedUrl.getHost(), AuthScope.ANY_PORT, AuthScope.ANY_REALM),
                    getCredentialSupplier().getCredentials(null, feedUrl.getHost()));
        }
        return client;
    }


    private boolean isHTTPS(URL feedUrl) {
        return feedUrl.getProtocol().equalsIgnoreCase("HTTPS");
    }

    /**
     * Test whether or not the feed is authenticated.
     *
     * @return true if the feed requires credentials, false if not.
     */
    public boolean testFeedAuthentication(String testURL) {
        //open new connection using HttpClient, check response code
        HttpMethod httpMethod = null;
        try {
            URL url = new URL(testURL);
            HttpClient httpClient = getHttpClient(url, -1);
            httpMethod = getHttpGetMethod(url);

            int statusCode = httpClient.executeMethod(httpMethod);
            if (statusCode == HttpStatus.SC_UNAUTHORIZED) {
                //log.info(testURL + " requires authentication");
                return true;
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        finally {
            if (httpMethod != null) {
                httpMethod.releaseConnection();
            }
        }
        return false;
    }

    /**
     * @param feedUrl
     * @param urlStr
     * @param method
     * @param feed
     * @return
     * @throws MalformedURLException
     */
    private SyndFeedInfo buildSyndFeedInfo(URL feedUrl, String urlStr, HttpMethod method,
            SyndFeed feed, int statusCode) throws MalformedURLException
    {
        SyndFeedInfo syndFeedInfo;
        syndFeedInfo = new SyndFeedInfo();

        // this may be different to feedURL because of 3XX redirects
        syndFeedInfo.setUrl(new URL(urlStr));
        syndFeedInfo.setId(feedUrl.toString());

        Header imHeader = method.getResponseHeader("IM");
        if (imHeader != null && imHeader.getValue().indexOf("feed") >= 0
                && isUsingDeltaEncoding())
        {
            FeedFetcherCache cache = getFeedInfoCache();
            if (cache != null && statusCode == 226) {
                // client is setup to use http delta encoding and the server supports it and has returned a delta encoded response
                // This response only includes new items
                SyndFeedInfo cachedInfo = cache.getFeedInfo(feedUrl);
                if (cachedInfo != null) {
                    SyndFeed cachedFeed = cachedInfo.getSyndFeed();

                    // set the new feed to be the orginal feed plus the new items
                    feed = combineFeeds(cachedFeed, feed);
                }
            }
        }

        Header lastModifiedHeader = method.getResponseHeader("Last-Modified");
        if (lastModifiedHeader != null) {
            syndFeedInfo.setLastModified(lastModifiedHeader.getValue());
        }

        Header eTagHeader = method.getResponseHeader("ETag");
        if (eTagHeader != null) {
            syndFeedInfo.setETag(eTagHeader.getValue());
        }

        syndFeedInfo.setSyndFeed(feed);

        return syndFeedInfo;
    }

    /**
     * @param urlStr
     * @param method
     * @return
     * @throws IOException
     * @throws HttpException
     * @throws FetcherException
     * @throws FeedException
     */
    private static SyndFeed retrieveFeed(String urlStr, HttpMethod method)
            throws IOException, HttpException, FetcherException, FeedException
    {

        InputStream stream = null;
        if ((method.getResponseHeader("Content-Encoding") != null) && ("gzip".equalsIgnoreCase(
                method.getResponseHeader("Content-Encoding").getValue())))
        {
            stream = new GZIPInputStream(method.getResponseBodyAsStream());
        }
        else {
            stream = method.getResponseBodyAsStream();
        }
        try {
            XmlReader reader = null;
            if (method.getResponseHeader("Content-Type") != null) {
                reader = new XmlReader(stream, method.getResponseHeader("Content-Type").getValue(),
                        true);
            }
            else {
                reader = new XmlReader(stream, true);
            }
            return new SyndFeedInput().build(reader);
        }
        finally {
            if (stream != null) {
                stream.close();
            }
        }
    }

    private SyndFeed getFeed(SyndFeedInfo syndFeedInfo, String urlStr, HttpMethod method,
            int statusCode) throws IOException, HttpException, FetcherException, FeedException
    {

        if (statusCode == HttpURLConnection.HTTP_NOT_MODIFIED && syndFeedInfo != null) {
            fireEvent(FetcherEvent.EVENT_TYPE_FEED_UNCHANGED, urlStr);
            return syndFeedInfo.getSyndFeed();
        }

        SyndFeed feed = retrieveFeed(urlStr, method);
        fireEvent(FetcherEvent.EVENT_TYPE_FEED_RETRIEVED, urlStr, feed);
        return feed;
    }

    public class BasicAuthCredentials implements HttpClientFeedFetcher.CredentialSupplier {

        String userpass;
        String user;
        String pass;

        public BasicAuthCredentials(String userpass) {
            this.userpass = userpass;
        }

        public BasicAuthCredentials(String user, String pass) {
            this.user = user;
            this.pass = pass;
        }

        public Credentials getCredentials(java.lang.String realm, java.lang.String host) {
            if (user != null && pass != null) {
                return new UsernamePasswordCredentials(user, pass);
            }
            else {
                return new UsernamePasswordCredentials(userpass);
            }
        }
    }
}
