package org.jivesoftware.plugins.instantFeeds;

import org.jivesoftware.util.Log;
import org.jivesoftware.openfire.XMPPServer;

import java.util.HashMap;
import java.util.Map;
import java.util.List;

/**
 * An xmlrpc class for receiving weblog pings per the weblogs.com XML-RPC specification:
 * <a href="http://www.weblogs.com/api.html">http://www.weblogs.com/api.html</a>.
 */
public class InstantFeedsXMLRPC {

    public InstantFeedsXMLRPC() {

    }

    /**
     * The implementation of the weblogs.com ping method specification.
     * @param name the name of site
     * @param url the URL of site or RSS feed
     * @return a map which contains two keys: 'flerror' and 'message'.
     */
    @SuppressWarnings("unchecked")
    public Map ping(String name, String url) {
        Log.debug("name=" + name + ",url=" + url);
        Map results = new HashMap();
        InstantFeedsPlugin plugin = (InstantFeedsPlugin) XMPPServer.getInstance().getPluginManager().getPlugin("instantfeeds");
        // the specification says that the second parameter of this method call could be either
        // the URL of an RSS feed or the URL of the website so eventually support needs to be added
        // for retrieving the feed based on the URL of the site (which can never be done with 100%
        // accuracy, but we can at least try
        Feed f = plugin.loadFeed(url);
        if (f != null) {
            plugin.addFeedUpdate(f);
        }
        // no feed, possibly a website, try to autodiscover the feed
        else {
            List<String> feeds = plugin.discoverFeeds(url);
            if (feeds != null && feeds.size() > 0) {
                plugin.loadFeed(feeds.get(0));
            }

        }

        results.put("flerror", false);
        results.put("message", "Thanks for the ping.");
        return results;
    }

    /**
     * The weblogs.com specification says that three parameters is actually not legal,
     * but wordpress uses three. 
     * @param name the name of site
     * @param url the URL of site or RSS feed
     * @param feed the URL of an RSS, RDF, or Atom feed
     * @return a map which contains two keys: 'flerror' and 'message'.
     */
    public Map extendedPing(String name, String url, String feed) {
        return extendedPing(name, url, null, feed, null);
    }

    /**
     * The mininum number of arguments for  the extendedPing method according to the
     * weblogs.com specification.
     * @param name the name of site
     * @param url the URL of site or RSS feed
     * @param changes the url of the page to be checked for changes
     * @param feed the URL of an RSS, RDF, or Atom feed
     * @return a map which contains two keys: 'flerror' and 'message'.
    */
    public Map extendedPing(String name, String url, String changes, String feed) {
        return extendedPing(name, url, null, feed, null);
    }

    /**
     * The implementation of the extendedPing method according to the weblogs.com
     * specification.  The changes parameter, url parameter and category parameter
     * are all ignored.
     * @param name the name of site
     * @param url the URL of site or RSS feed
     * @param changes the url of the page to be checked for changes
     * @param feed the URL of an RSS, RDF, or Atom feed
     * @param category a name (or "tag") categorizing your site content
     * @return a map which contains two keys: 'flerror' and 'message'.
     */
    @SuppressWarnings("unchecked")
    public Map extendedPing(String name, String url, String changes, String feed, String category) {
        Log.debug("name=" + name + ",url=" + url + ",changes=" + changes +",feed=" + feed + ",category=" + category);
        Map results = new HashMap();
        InstantFeedsPlugin plugin = (InstantFeedsPlugin) XMPPServer.getInstance().getPluginManager().getPlugin("instantfeeds");
        Feed f = plugin.loadFeed(feed);
        if (f != null) {
            plugin.addFeedUpdate(f);
        }


        results.put("flerror", false);
        results.put("message", "Thanks for the ping.");
        return results;
    }





}